==> resource_planner-0.4.0.gem: app/assets/javascripts/resource_planner/fullcalendar_timeline/fullcalendar_timeline_resource_manipulation.js <==
//
// FullcalendarTimeline - Resource manipulation controls
//

// Initialise timeline manipulation controls
FullcalendarTimeline.prototype.initialiseResourceManipulationControls = function () {
  // Mechanism to pass (this) to the handlers
  var fullCalendarTimelineObject = this;

  this._selectedResources = [];

  // TODO: selectedBackgroundColour values should come from the application or database configuration
  //  values are ordered: label (resource), body (timeline)
  this._unselectedBackgroundColour = ['rgba(0, 0, 0, 0)', 'rgba(0, 0, 0, 0)'];
  //this._selectedBackgroundColour = ['LightSeaGreen', 'DarkSeaGreen'];
  this._selectedBackgroundColour = ['LightSeaGreen', 'rgba(0, 0, 0, 0)'];
  this._conflictBackgroundColour = ['IndianRed', 'LightSalmon'];

  // Get existing events and resource restrictions
  $.when(
    $.ajax({ url: '/resource_planner/resource_events.json', async: true }),
    $.ajax({ url: '/resource_planner/resource_restrictions.json', async: true })
  ).done(function (a1, a2) {
    fullCalendarTimelineObject._existingEvents = a1[0];
    fullCalendarTimelineObject._resourceRestrictions = a2[0];
  });

  // for resourceSelectedHandler() initialisation, see resourceRender callback in fullcalendar initialisation
  this.renderSelectedResources();
}

FullcalendarTimeline.prototype.resourceSelectedHandler = function (renderInfo) {
  var resourceId = renderInfo.resource.id

  var index = this._selectedResources.map(function (e) { return e.resourceId; }).indexOf(resourceId);

  if (index == -1) {
    // Add resource to _selectedResources
    var resourceData = {};
    resourceData.resourceId = resourceId;
    resourceData.isInConflict = false;
    resourceData.conflictingEvents = [];
    resourceData.conflictingRestrictions = [];

    this._selectedResources.push(resourceData);
  } else {
    // Remove resource from _selectedResources
    this._selectedResources = this._selectedResources.filter(function (value, index, arr) {
      return value.resourceId != resourceId;
    });
  }

  this.renderSelectedResources();

  // labelTds.css('background', 'blue');
  // bodyTds.css('background', 'yellow');
  // resource_id = labelTds.parent().attr("data-resource-id");
  // labelTds.eq(0).find('.fc-cell-content')
  //   .append(
  //     $('<strong>('+resource_id+')</strong>')
  //     //$('<strong>(?)</strong>')
  //     //.popover({
  //     //  title: resourceObj.title,
  //     //  content: 'test!',
  //     //  trigger: 'hover',
  //     //  placement: 'bottom',
  //     //  container: 'body'
  //     //})
  //   );
}

FullcalendarTimeline.prototype.renderSelectedResources = function () {
  // Clear all highlighting on resources
  this._timelineViewWrapper.find('.fc-resource-area [data-resource-id]').css('background-color', this._unselectedBackgroundColour[0]);
  this._timelineViewWrapper.find('.fc-time-area [data-resource-id]').css('background-color', this._unselectedBackgroundColour[1]);

  // Check for conflicts
  this.flagResourceConflicts();

  // Render event conflicts
  this.displayHighlighterConflicts();

  // Render resource conflicts
  for (var i = 0; i < this._selectedResources.length; i++) {
    var resourceAreaColour = (this._selectedResources[i].isInConflict) ? this._conflictBackgroundColour[0] : this._selectedBackgroundColour[0];
    var timeAreaColour = (this._selectedResources[i].isInConflict) ? this._conflictBackgroundColour[1] : this._selectedBackgroundColour[1];

    this._timelineViewWrapper.find('.fc-resource-area [data-resource-id="' + this._selectedResources[i].resourceId + '"]').css('background-color', resourceAreaColour);

    this._timelineViewWrapper.find('.fc-time-area [data-resource-id="' + this._selectedResources[i].resourceId + '"]').css('background-color', timeAreaColour);
  }
}

FullcalendarTimeline.prototype.flagResourceConflicts = function () {

  // get currently planned event list (eventArray)
  var eventsArray = this._eventsArray

  // see where they conflict and flag the resource
  if (typeof eventsArray !== 'undefined') {

    // Clear active conflicts on the highlighter bar
    this.clearHighlighterConflicts();

    for (var selectedResourceindex = 0; selectedResourceindex < this._selectedResources.length; selectedResourceindex++) {

      this._selectedResources[selectedResourceindex].isInConflict = false;
      this._selectedResources[selectedResourceindex].conflictingEvents = [];
      this._selectedResources[selectedResourceindex].conflictingRestrictions = [];

      for (var eventArrayIndex = 0; eventArrayIndex < eventsArray.length; eventArrayIndex++) {

        for (var i = 0; i < this._existingEvents.length; i++) {
          if (this._existingEvents[i].resourceIds.indexOf(this._selectedResources[selectedResourceindex].resourceId) != -1) {

            var resourceQuantity = parseInt(this._selectedResources[selectedResourceindex].resourceId.split("-")[2], 10);

            // TODO: Arbitrate for resourceQuantity > 1
            if (resourceQuantity > 0 &&
              (dateCheck(this._existingEvents[i].start, this._existingEvents[i].end, eventsArray[eventArrayIndex].startDate)
                || dateCheck(this._existingEvents[i].start, this._existingEvents[i].end, eventsArray[eventArrayIndex].endDate)
              )
            ) {
              this._selectedResources[selectedResourceindex].isInConflict = true;
              var conflictingEvent = {};
              conflictingEvent.id = this._existingEvents[i].id;
              conflictingEvent.name = this._existingEvents[i].title;
              this._selectedResources[selectedResourceindex].conflictingEvents.push(conflictingEvent);
              eventsArray[eventArrayIndex].isInConflict = true;
            }
          }
        }

        for (var j = 0; j < this._resourceRestrictions.length; j++) {
          if (this._resourceRestrictions[j].resourceIds.indexOf(this._selectedResources[selectedResourceindex].resourceId) != -1) {
            if (dateCheck(this._resourceRestrictions[j].start, this._resourceRestrictions[j].end, eventsArray[eventArrayIndex].startDate)
              || dateCheck(this._resourceRestrictions[j].start, this._resourceRestrictions[j].end, eventsArray[eventArrayIndex].endDate)
            ) {
              this._selectedResources[selectedResourceindex].isInConflict = true;
              var conflictingRestriction = {};
              conflictingRestriction.id = this._existingEvents[j].id;
              conflictingRestriction.name = this._existingEvents[j].title;
              this._selectedResources[selectedResourceindex].conflictingRestrictions.push(conflictingRestriction);
              eventsArray[eventArrayIndex].isInConflict = true;
            }
          }
        }

      }
    }

    // Update the 'global' eventsArray
    this._eventsArray = eventsArray;
  }

}

==> resource_planner-0.4.1.gem: app/assets/javascripts/resource_planner/fullcalendar_timeline/fullcalendar_timeline_resource_manipulation.js <==
//
// FullcalendarTimeline - Resource manipulation controls
//

// Initialise timeline manipulation controls
FullcalendarTimeline.prototype.initialiseResourceManipulationControls = function () {
  // Mechanism to pass (this) to the handlers
  var fullCalendarTimelineObject = this;

  this._selectedResources = [];

  // TODO: selectedBackgroundColour values should come from the application or database configuration
  //  values are ordered: label (resource), body (timeline)
  this._unselectedBackgroundColour = ['rgba(0, 0, 0, 0)', 'rgba(0, 0, 0, 0)'];
  //this._selectedBackgroundColour = ['LightSeaGreen', 'DarkSeaGreen'];
  this._selectedBackgroundColour = ['LightSeaGreen', 'rgba(0, 0, 0, 0)'];
  this._conflictBackgroundColour = ['IndianRed', 'LightSalmon'];

  // Get existing events and resource restrictions
  $.when(
    $.ajax({ url: '/resource_planner/resource_events.json', async: true }),
    $.ajax({ url: '/resource_planner/resource_restrictions.json', async: true })
  ).done(function (a1, a2) {
    fullCalendarTimelineObject._existingEvents = a1[0];
    fullCalendarTimelineObject._resourceRestrictions = a2[0];
  });

  // for resourceSelectedHandler() initialisation, see resourceRender callback in fullcalendar initialisation
  this.renderSelectedResources();
}

FullcalendarTimeline.prototype.resourceSelectedHandler = function (renderInfo) {
  var resourceId = renderInfo.resource.id

  var indexOfResourceId = this._selectedResources.map(function (e) { return e.resourceId; }).indexOf(resourceId);

  if (indexOfResourceId == -1) {
    // Add resource to _selectedResources
    var resourceData = {};
    resourceData.resourceId = resourceId;
    resourceData.isInConflict = false;
    resourceData.conflictingEvents = [];
    resourceData.conflictingRestrictions = [];

    this._selectedResources.push(resourceData);
  } else {
    // Remove resource from _selectedResources
    this._selectedResources = this._selectedResources.filter(function (element, index, array) {
      return element.resourceId != resourceId;
    });
  }

  this.renderSelectedResources();

  // labelTds.css('background', 'blue');
  // bodyTds.css('background', 'yellow');
  // resource_id = labelTds.parent().attr("data-resource-id");
  // labelTds.eq(0).find('.fc-cell-content')
  //   .append(
  //     $('<strong>('+resource_id+')</strong>')
  //     //$('<strong>(?)</strong>')
  //     //.popover({
  //     //  title: resourceObj.title,
  //     //  content: 'test!',
  //     //  trigger: 'hover',
  //     //  placement: 'bottom',
  //     //  container: 'body'
  //     //})
  //   );
}

FullcalendarTimeline.prototype.renderSelectedResources = function () {
  // Clear all highlighting on resources
  this._timelineViewWrapper.find('.fc-resource-area [data-resource-id]').css('background-color', this._unselectedBackgroundColour[0]);
  this._timelineViewWrapper.find('.fc-time-area [data-resource-id]').css('background-color', this._unselectedBackgroundColour[1]);

  // Check for conflicts
  this.flagResourceConflicts();

  // Render event conflicts
  this.displayHighlighterConflicts();

  // Render resource conflicts
  for (var i = 0; i < this._selectedResources.length; i++) {
    var resourceAreaColour = (this._selectedResources[i].isInConflict) ? this._conflictBackgroundColour[0] : this._selectedBackgroundColour[0];
    var timeAreaColour = (this._selectedResources[i].isInConflict) ? this._conflictBackgroundColour[1] : this._selectedBackgroundColour[1];

    this._timelineViewWrapper.find('.fc-resource-area [data-resource-id="' + this._selectedResources[i].resourceId + '"]').css('background-color', resourceAreaColour);

    this._timelineViewWrapper.find('.fc-time-area [data-resource-id="' + this._selectedResources[i].resourceId + '"]').css('background-color', timeAreaColour);
  }
}

FullcalendarTimeline.prototype.flagResourceConflicts = function () {

  // get currently planned event list (eventArray)
  var eventsArray = this._eventsArray

  // see where they conflict and flag the resource
  if (typeof eventsArray !== 'undefined') {

    // Clear active conflicts on the highlighter bar
    this.clearHighlighterConflicts();

    for (var selectedResourceindex = 0; selectedResourceindex < this._selectedResources.length; selectedResourceindex++) {

      this._selectedResources[selectedResourceindex].isInConflict = false;
      this._selectedResources[selectedResourceindex].conflictingEvents = [];
      this._selectedResources[selectedResourceindex].conflictingRestrictions = [];

      for (var eventArrayIndex = 0; eventArrayIndex < eventsArray.length; eventArrayIndex++) {

        var totalResourceAllocatedForThisEventUnit = 0;

        for (var existingEventIndex = 0; existingEventIndex < this._existingEvents.length; existingEventIndex++) {

          var resourceIdIndex = this._existingEvents[existingEventIndex].resourceIds.indexOf(this._selectedResources[selectedResourceindex].resourceId);

          if (resourceIdIndex != -1) {
            var resourceId = this._existingEvents[existingEventIndex].resourceIds[resourceIdIndex];
            var requestedResourceQuantity = parseInt(this._timelineViewWrapper.find('.fc-resource-area [data-resource-id="' + resourceId + '"]').find('input').val(), 10);
            var resourceAllocation = this._existingEvents[existingEventIndex].resourceAllocation[resourceIdIndex];
            var totalResourceQuantity = parseInt(resourceAllocation.maximumAvailable, 10);
            var allocatedResourceQuantity = parseInt(resourceAllocation.allocated, 10);

            totalResourceAllocatedForThisEventUnit += allocatedResourceQuantity;
            var requestedResourceQuantityIsNotAvailable = ((totalResourceAllocatedForThisEventUnit + requestedResourceQuantity) > totalResourceQuantity);

            if (
              totalResourceQuantity > 0 && requestedResourceQuantityIsNotAvailable &&
              (dateCheck(this._existingEvents[existingEventIndex].start, this._existingEvents[existingEventIndex].end, eventsArray[eventArrayIndex].startDate)
                || dateCheck(this._existingEvents[existingEventIndex].start, this._existingEvents[existingEventIndex].end, eventsArray[eventArrayIndex].endDate))
            ) {
              var conflictingEvent = {};
              conflictingEvent.id = this._existingEvents[existingEventIndex].id;
              conflictingEvent.name = this._existingEvents[existingEventIndex].title;
              this._selectedResources[selectedResourceindex].conflictingEvents.push(conflictingEvent);
              this._selectedResources[selectedResourceindex].isInConflict = true;

              eventsArray[eventArrayIndex].isInConflict = true;
            }
          }
        }

        for (var resourceRestrictionIndex = 0; resourceRestrictionIndex < this._resourceRestrictions.length; resourceRestrictionIndex++) {
          if (this._resourceRestrictions[resourceRestrictionIndex].resourceIds.indexOf(this._selectedResources[selectedResourceindex].resourceId) != -1) {
            if (dateCheck(this._resourceRestrictions[resourceRestrictionIndex].start, this._resourceRestrictions[resourceRestrictionIndex].end, eventsArray[eventArrayIndex].startDate)
              || dateCheck(this._resourceRestrictions[resourceRestrictionIndex].start, this._resourceRestrictions[resourceRestrictionIndex].end, eventsArray[eventArrayIndex].endDate)
            ) {
              var conflictingRestriction = {};
              conflictingRestriction.id = this._existingEvents[resourceRestrictionIndex].id;
              conflictingRestriction.name = this._existingEvents[resourceRestrictionIndex].title;
              this._selectedResources[selectedResourceindex].conflictingRestrictions.push(conflictingRestriction);
              this._selectedResources[selectedResourceindex].isInConflict = true;

              eventsArray[eventArrayIndex].isInConflict = true;
            }
          }
        }

      }
    }

    // Update the 'global' eventsArray
    this._eventsArray = eventsArray;
  }

}
